package ru.pif.mobile.android;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONObject;

import ru.pif.mobile.android.speech.SpeechHandler;
import android.app.Activity;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.database.Cursor;
import android.media.AudioManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Vibrator;
import android.provider.ContactsContract;
import android.provider.ContactsContract.Contacts;
import android.provider.ContactsContract.Data;
import android.speech.RecognizerIntent;
import android.util.Base64;

import com.adobe.fre.FREContext;
import com.adobe.fre.FREFunction;
import com.adobe.fre.FREObject;
import com.adobe.fre.FREWrongThreadException;

public class ContextExt extends FREContext {

	static public ContextExt activite;
	private Vibrator serviceVibrate = null;
	private AudioManager serviceAudio = null;
	private final String VERSION_ANE = "30.07-01:28";

	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	@Override
	public Map<String, FREFunction> getFunctions() {
		activite = this;
		Map<String, FREFunction> functionMap = new HashMap<String, FREFunction>();

		functionMap.put("openWithDefaultApplication", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				String url = passedArgs[0].getAsString();
				String mime = passedArgs[1].getAsString();
				File file = new File(url);
				if (file.exists()) {
					Intent intent = new Intent();
					intent.setAction(android.content.Intent.ACTION_VIEW);
					intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
					intent.setDataAndType(Uri.fromFile(file), mime); // "application/pdf"
					getActivity().startActivity(intent);
					return createResult(true);
				}
				return createResult("no file");
			}
		});

		functionMap.put("getContacts", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				Integer type = passedArgs[0].getAsInt();
				String filtr = passedArgs[1].getAsString();
				String[] str = (filtr.equals("") ? null : filtr.split(","));

				JSONObject result = new JSONObject();
				result.put("str", str);
				result.put("type", type);
				Cursor cursor = null;

				switch (type) {
				case 0: {
					cursor = getActivity().getContentResolver().query(
							Data.CONTENT_URI, str, null, null, null);
					break;
				}
				case 1: {
					cursor = getActivity().getContentResolver().query(
							ContactsContract.CommonDataKinds.Phone.CONTENT_URI,
							str, null, null, null);
					break;
				}
				case 2: {
					cursor = getActivity().getContentResolver().query(
							ContactsContract.CommonDataKinds.Email.CONTENT_URI,
							str, null, null, null);
					break;
				}
				case 3: {
					Integer contactId = Integer.parseInt(str[0]);
					result.put("contactId", contactId);
					Uri contactUri = ContentUris.withAppendedId(
							Contacts.CONTENT_URI, contactId);
					Uri photoUri = Uri.withAppendedPath(contactUri,
							Contacts.Photo.CONTENT_DIRECTORY);
					cursor = getActivity().getContentResolver().query(photoUri,
							null, null, null, null);
					break;
				}
				}

				getActivity().startManagingCursor(cursor);

				JSONArray list = new JSONArray();
				if (cursor.getCount() > 0) {
					while (cursor.moveToNext()) {
						JSONObject row = new JSONObject();
						for (int i = 0; i < cursor.getColumnCount(); i++) {
							try {
								row.put(cursor.getColumnName(i),
										cursor.getString(i));
							} catch (Exception e) {
								String s = Base64.encodeToString(
										cursor.getBlob(i), Base64.DEFAULT);
								row.put(cursor.getColumnName(i), s);
							}
						}
						list.put(row);
					}
				}
				result.put("result", list);
				return createResult(result.toString());
			}
		});

		functionMap.put("speechListen", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {

				String promt = passedArgs[0].getAsString();
				String lang = passedArgs[1].getAsString();
				int count = passedArgs[2].getAsInt();

				SpeechHandler.speechIntent = new Intent(
						RecognizerIntent.ACTION_RECOGNIZE_SPEECH);
				SpeechHandler.speechIntent.putExtra(
						RecognizerIntent.EXTRA_PROMPT, promt);
				SpeechHandler.speechIntent.putExtra(
						RecognizerIntent.EXTRA_MAX_RESULTS, count);
				SpeechHandler.speechIntent.putExtra(
						RecognizerIntent.EXTRA_LANGUAGE, lang);

				Intent in = new Intent(getActivity(), SpeechHandler.class);
				getActivity().startActivity(in);
				return createResult(lang + count);
			}
		});

		functionMap.put("speechisSupported", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				// Check to see if a recognition activity is present
				PackageManager pm = getActivity().getPackageManager();
				List<ResolveInfo> activities = pm
						.queryIntentActivities(new Intent(
								RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
				if (activities.size() != 0) {
					return createResult(true);
				} else {
					return createResult(false);
				}
			}
		});

		// vibrate
		functionMap.put("initVibrate", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				serviceVibrate = (Vibrator) getActivity().getSystemService(
						Context.VIBRATOR_SERVICE);
				return createResult(serviceVibrate != null);
			}
		});
		functionMap.put("runVibrate", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				FREObject fro = passedArgs[0];
				int duration = fro.getAsInt();
				if (serviceVibrate != null) {
					serviceVibrate.vibrate(duration);
					return createResult(true);
				}
				return createResult(false);
			}
		});

		// audio
		functionMap.put("initAudio", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				serviceAudio = (AudioManager) getActivity().getSystemService(
						Context.AUDIO_SERVICE);
				return createResult(serviceAudio != null);
			}
		});
		functionMap.put("isSpeaker", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				return createResult(serviceAudio.isSpeakerphoneOn());
			}
		});
		functionMap.put("isVoice", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				dispatchStatusEventAsync("isVoice", "222");
				Intent i = getActivity().getIntent();
				if (i == null)
					createResult(1);
				Bundle extras = i.getExtras();
				if (extras == null)
					createResult(2);
				String t = extras.getString(RecognizerIntent.EXTRA_PROMPT);
				return createResult(t);
			}
		});
		functionMap.put("startRecognition", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				// создаем Intent с действием
				// RecognizerIntent.ACTION_RECOGNIZE_SPEECH
				Intent intent = new Intent(
						RecognizerIntent.ACTION_RECOGNIZE_SPEECH);

				// текстовая подсказка пользователю
				intent.putExtra(RecognizerIntent.EXTRA_PROMPT,
						"Голосовой поиск Inforino");
				// модель распознавания оптимальная для распознавания коротких
				// фраз-поисковых запросов
				intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
						RecognizerIntent.LANGUAGE_MODEL_WEB_SEARCH);
				// количество результатов, которое мы хотим получить, в данном
				// случае хотим только первый - самый релевантный
				intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1);
				dispatchStatusEventAsync("1!", "222");
				// getActivity().startActivity(intent);
				// стартуем Activity и ждем от нее результата
				// getActivity().startActivityFromChild(MainActivity._this,
				// intent, 25);
				getActivity().startActivityForResult(intent, 25);
				return createResult(isSpeechRecognitionActivityPresented(getActivity()));
			}
		});
		functionMap.put("setSpeakerphoneOn", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				Boolean on = passedArgs[0].getAsBool();
				if (serviceAudio != null) {
					if (on)
						serviceAudio.setMode(AudioManager.MODE_NORMAL);
					else
						serviceAudio.setMode(AudioManager.MODE_IN_CALL);

					serviceAudio.setSpeakerphoneOn(on);
					return createResult(serviceAudio.isSpeakerphoneOn());
				}
				return createResult();
			}
		});

		// system
		functionMap.put("build", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				final String SEP = "!;";
				String dataAr = "VERSION_RELEASE=" + Build.VERSION.RELEASE
						+ SEP + "VERSION_INCREMENTAL="
						+ Build.VERSION.INCREMENTAL + SEP + "VERSION_SDK="
						+ Build.VERSION.SDK + SEP + "BOARD=" + Build.BOARD
						+ SEP + "BRAND=" + Build.BRAND + SEP + "DEVICE="
						+ Build.DEVICE + SEP + "FINGERPRINT="
						+ Build.FINGERPRINT + SEP + "HOST=" + Build.HOST + SEP
						+ "ID=" + Build.ID + SEP + "VERSION_ANE=" + VERSION_ANE;
				return createResult(dataAr);
			}
		});

		functionMap.put("test", new FunctionHelper() {
			@Override
			public FREObject invoke(FREContext context, FREObject[] passedArgs)
					throws Exception {
				return createResult(80);
			}
		});

		// functionMap.put("callServiceFunction", new FunctionHelper()
		// {
		// @Override
		// public FREObject invoke(FREContext context, FREObject[] passedArgs)
		// throws Exception
		// {
		// FREObject fro = passedArgs[0];
		// String serviceName = fro.getAsString();
		// fro = passedArgs[1];
		// String methodName = fro.getAsString();
		// Object service=getActivity().getSystemService(serviceName);
		// switch(passedArgs.length){
		// case 2:{
		// //service[methodName]();
		// break;
		// }
		// case 3:{
		// //service[methodName](passedArgs[2].getAsString());
		// break;
		// }
		// }
		// return createResult("25");
		// }
		// });

		return functionMap;
	}

	public FREObject createResult(String value) {
		FREObject result = null;
		try {
			result = FREObject.newObject(value);
		} catch (FREWrongThreadException e) {
			e.printStackTrace();
		}
		return result;
	}

	public FREObject createResult(Integer value) {
		FREObject result = null;
		try {
			result = FREObject.newObject(value);
		} catch (FREWrongThreadException e) {
			e.printStackTrace();
		}
		return result;
	}

	public FREObject createResult(Boolean value) {
		FREObject result = null;
		try {
			result = FREObject.newObject(value);
		} catch (FREWrongThreadException e) {
			e.printStackTrace();
		}
		return result;
	}

	public FREObject createResult() {
		FREObject result = null;
		return result;
	}

	private boolean isSpeechRecognitionActivityPresented(Activity ownerActivity) {
		try {
			// получаем экземпляр менеджера пакетов
			PackageManager pm = ownerActivity.getPackageManager();
			// получаем список Activity способных обработать запрос на
			// распознавание
			List<ResolveInfo> activities = pm.queryIntentActivities(new Intent(
					RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);

			if (activities.size() != 0) { // если список не пустой
				return true; // то умеем распознавать речь
			}
		} catch (Exception e) {

		}
		return false; // не умеем распознавать речь
	}
}
